# coding=utf-8
"""
This file is part of GeoRemindMe.

GeoRemindMe is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

GeoRemindMe is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with GeoRemindMe.  If not, see <http://www.gnu.org/licenses/>.


"""
#===============================================================================
# Based python-oauth: https://github.com/leah/python-oauth
#===============================================================================
from random import randrange
from time import time
import urllib

def escape(s):
    """Escape a URL including any /."""
    return urllib.quote(s, safe='~')

class OAuthToken(object):
    key = None
    secret = None
    callback_confirmed = None
    verifier = None
    
    def __init__(self, key, secret, verifier=None):
        self.key = escape(key)
        self.secret = escape(secret)
        if verifier:
            self.verifier = verifier
    
    @classmethod
    def from_response(cls, response=None):
        from urlparse import parse_qs
        params = parse_qs(response, keep_blank_values=False)
        key = params['oauth_token'][0]
        secret = params['oauth_token_secret'][0]
        token = OAuthToken(key, secret)
        try:
            token.callback_confirmed = params['oauth_callback_confirmed'][0]
            token.verifier = params['oauth_verifier'][0]
        except KeyError:
            pass # 1.0, no callback confirmed.
        return token
    



class OAuthRequest(object):
    '''
    OAUTH AUTHENTICATION FLOW: http://a0.twimg.com/images/dev/oauth_diagram.png
    Initial headers parameters:
        oauth_consumer_key
        oauth_signature_method
        oauth_signature: signature 
        oauth_nonce: random string
        oauth_timestamp
        oauth_version: (optional)
        oauth_callback: url where the provider notify the oauth_token
    '''
    parameters = {}
    version = '1.0'
    http_method = 'GET'
    
    def __init__(self, http_method, http_url, params):
        self.http_method = http_method
        self.http_url = http_url
        self.parameters = params
        
    def normalize_http_method(self):
        return escape(self.http_method.upper())
    
    def normalize_http_url(self):
        '''
        remove port, query, etc. from http url
        '''
        from urlparse import urlparse
        #ParseResult(scheme='http', netloc='www.cwi.nl:80', path='/%7Eguido/Python.html', params='', query='', fragment='')
        scheme, address, path = urlparse(self.http_url)[:3]#we only need the first three parts
        # Exclude default port numbers.
        if scheme == 'http' and address[-3:] == ':80':
            address = address[:-3]
        elif scheme == 'https' and address[-4:] == ':443':
            address = address[:-4]
        return escape('%s://%s%s' % (scheme, address, path))
    
    def normalize_parameters(self):
        '''
        returns parameters as a url valid string
        '''    
        temp = self.parameters
        if 'oauth_signature' in temp:
            del temp['oauth_signature']
        
        key_values = [(escape(k), escape(v)) for k,v in temp.iteritems()]
        key_values.sort()
        return escape('&'.join(['%s=%s' % (k, v) for k, v in key_values]))
    
    @classmethod
    def get_request_token(cls, oauth_consumer=None, callback=None, http_method='GET', http_url=None):
        '''
        builds the standard consumer request to get a token
        '''
        
        base = {
                'oauth_consumer_key': oauth_consumer.key,
                'oauth_timestamp':generate_timestamp(),
                'oauth_nonce': generate_nonce(),
                'oauth_version': OAuthRequest.version,
                }
        
        if callback:
            base['oauth_callback'] = callback
            
        r = OAuthRequest(http_method, http_url, base)
        r.sign_request(oauth_consumer)
        return r
    
    @classmethod
    def authorize_request_token(cls, token=None, callback=None, http_method='GET', http_url=None):
        '''
        with the token provided you need to redirect the user to the provider web, with the authorization url
        '''
        base = {
                'oauth_token' : token.key
                }
        if callback:
            base['oauth_callback'] = callback
            
        return OAuthRequest(http_method, http_url, base)
        
    
    @classmethod
    def get_from_provider(cls, oauth_consumer=None, token=None, http_method='GET', http_url=None, parameters = None, callback=None):
        '''
        with the oauth_verifier, ask provider a access token with the access token url
        or send request to the resource url
        '''
        request = cls.get_request_token(oauth_consumer=oauth_consumer, http_method=http_method, http_url=http_url, callback=callback)
        request.parameters['oauth_token'] = token.key
        if parameters:
            request.parameters.update(parameters)
        if token.verifier:
            request.parameters['oauth_verifier'] = token.verifier
        
        request.sign_request(oauth_consumer, token)
        return request

    
    def sign_request(self, oauth_consumer=None, token=None):
        """Set the signature parameter to the result of build_signature."""
        # Set the signature method.
        from signatures.HMAC_SHA1 import HMAC_SHA1 as signature
        sig = signature()
        self.parameters['oauth_signature_method'] = sig.get_name()
        # Set the signature.
        self.parameters['oauth_signature'] = sig.build_signature(self, oauth_consumer, token)
        
def generate_timestamp():
    '''
    return seconds since epoch
    '''
    return str(int(time()))

def generate_nonce():
    '''
    generate a unique nonce
    '''
    from django.utils.hashcompat import md5_constructor
    return str(md5_constructor("%s%s"
                                  % (randrange(0,18446744073709551616L), time())
                                    ).hexdigest()[:8])
    